/**
 * Card Class represents A card and it is super Class
 * Card has name and color that can be null
 * there is a method to check it is possible to put card or not
 *
 * @author zahra_SeyedFatehi
 * @version 0.0
 */

import java.util.ArrayList;

public abstract class Card {
    protected String name;
    protected String color;
    private int score;
    public static final String set = "\u001B[0m";
    public static final String green = "\u001B[32m";
    public static final String red = "\u001B[31m";
    public static final String yellow = "\u001B[33m";
    public static final String blue = "\u001B[34m";

    /**
     * get color of a card and it can be null for wild Cards
     * @return color of a card
     */
    public String getColor() {
        return color;
    }

    /**
     * @param color set the color of a card
     */
    public void setColor(String color) {
        this.color = color;
    }

    /**
     *
     * @param score set score in subClasses
     */
    public Card(int score) {
        this.score = score;
    }

    /**
     * get score of a card
     * @return score field
     */
    public int getScore() {
        return score;
    }

    /**
     *
     * @param score set score
     */
    public void setScore(int score) {
        this.score = score;
    }


    @Override
    public abstract String toString();

    /**
     * show the shape of a card
     */
    public abstract void print();

    /**
     *
     * @param chosenUser is who has the turn
     * @param numOfChosenCard is the number of chosen card in arrayList
     * @param users a list of players
     * @param remainingCards a list of cards in resource
     * @param upperCard is the top card
     * @return the chosen who has the turn
     */
    public abstract User act(User chosenUser, int numOfChosenCard ,ArrayList<User> users, ArrayList<Card> remainingCards, Card upperCard);

    /**
     * check it is possible to put card or not
     * @param chosenUser is who has the turn
     * @param upperCard is the top card
     * @return true if it is available to put card and false if it is not
     */
    public boolean isAvailableToPut( User chosenUser, Card upperCard) {

        if (this.getScore() < 10 && upperCard.getScore() < 10){
            NumericalCard upper = (NumericalCard)upperCard;
            NumericalCard chosen = (NumericalCard)this;
            if (color.equals(upper.getColor()) || chosen.getNumber() == upper.getNumber()){
                return true;
            }
            else
                return false;
        }
        else if (this.getScore() < 50 && upperCard.getScore() < 50) {
            if ((this instanceof SkipCard && upperCard instanceof SkipCard) ||(this instanceof ReverseCard && upperCard instanceof ReverseCard)){
                //you can put a reverse card after a reverse card
                //you can put a skip card after a skip card
                return true;
            }
            ColoredCard upper = (ColoredCard) upperCard;//if color of a ColoredCard be as color of upper Card you can put it
            if (this.getColor().equals(upper.getColor()))
                return true;
            else
                return false;
        } else {
            for (Card card : chosenUser.getUserCards()) {
                if (card instanceof WildCard){// you can not put a wild card after a wild card
                    continue;
                }
                else {//if there is another card to choose you can not choose wild card
                    if (card.isAvailableToPut( chosenUser , upperCard)) {
                        return false;
                    }
                }
            }
            return true;
        }
    }
}
